# -*- coding: utf-8 -*-
from datetime import datetime

from HTMLParser import HTMLParseError

from django.contrib import admin
from django.conf import settings
from django.conf.urls.defaults import url
from django.forms.widgets import PasswordInput
from django.utils.translation import ugettext_lazy as _
from django.dispatch import Signal
from django.conf.urls.defaults import patterns
from django.core.urlresolvers import reverse
from django.shortcuts import get_object_or_404, render_to_response
from django import forms
from django.db.models import Q
from django.contrib.admin.options import IncorrectLookupParameters

from jumbo.admin.options import ExtendibleModelAdmin, SettingsAdmin
from jumbo.forms.fields import JumboDateField

from newsletter.models import Newsletter, Attachment, Contact, Settings,\
    LinkContactMailingStatus, WorkGroup, ContactExtraFields
from newsletter.models import MailingList, ContactMailingStatus, BounceAccount
from newsletter.models import SMTPServer, Credit, Link
from newsletter.mailer import Mailer
from newsletter.utils.vcard import vcard_contacts_export_response
from newsletter.settings import USE_WORKGROUPS, CAN_USE_PREMAILER
from newsletter.utils.workgroups import request_workgroups_newsletters_pk,\
    request_workgroups_mailinglists_pk, request_workgroups,\
    request_workgroups_contacts_pk
from newsletter.utils.premailer import Premailer, PremailerError
from newsletter.utils.excel import ExcelResponse
from django.http import HttpResponseRedirect
from newsletter.utils.importation import import_dispatcher
from django.contrib.admin.views.main import ChangeList
from django.template.context import RequestContext
from django.core.exceptions import ValidationError

admin.site.register(Settings, SettingsAdmin)

##############################
# NEWSLETTER                 #
##############################
class AttachmentAdminInline(admin.TabularInline):
    model = Attachment
    extra = 1
    fieldsets = ((None, {'fields': (('title', 'file_attachment'))}),)

class NewsletterAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("newsletter/css/admin_newsletter.css",)}

    #date_hierarchy = 'creation_date'
    list_display = (
        'status', 'title', 'mailing_list', 'server',
        'sending_date', 'creation_date', 'modification_date',
        'historic_link', 'statistics_link',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    #list_filter = ('status', 'sending_date', 'creation_date', 'modification_date')
    #search_fields = ('title', 'content', 'header_sender', 'header_reply')
    filter_horizontal = ['test_contacts']
    fieldsets = (
        (None, {'fields': ('title', 'content',)}),
        (_('Receivers'), {'fields': ('mailing_list', 'test_contacts',)}),
        (_('Sending'), {'fields': ('sending_date', 'status',)}),
        (_('Miscellaneous'), {
            'fields': (
                'server', 'bouncer', 'header_sender', 'header_reply', 'slug'
            ),
            'classes': ('collapse',)
        }),)
    prepopulated_fields = {'slug': ('title',)}
    inlines = (AttachmentAdminInline,)
    actions = ['send_mail_test', 'make_ready_to_send', 'make_cancel_sending']

    def get_actions(self, request):
        actions = super(NewsletterAdmin, self).get_actions(request)
        if not request.user.has_perm('newsletter.can_change_status'):
            del actions['make_ready_to_send']
            del actions['make_cancel_sending']
        return actions

    def queryset(self, request):
        queryset = super(NewsletterAdmin, self).queryset(request)
        if not request.user.is_superuser and USE_WORKGROUPS:
            newsletters_pk = request_workgroups_newsletters_pk(request)
            queryset = queryset.filter(pk__in=newsletters_pk)
        return queryset

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == 'mailing_list' and \
               not request.user.is_superuser and USE_WORKGROUPS:
            mailinglists_pk = request_workgroups_mailinglists_pk(request)
            kwargs['queryset'] = MailingList.objects.filter(pk__in=mailinglists_pk)
            return db_field.formfield(**kwargs)
        return super(NewsletterAdmin, self).formfield_for_foreignkey(
            db_field, request, **kwargs)

    def formfield_for_choice_field(self, db_field, request, **kwargs):
        if db_field.name == 'status' and \
               not request.user.has_perm('newsletter.can_change_status'):
            kwargs['choices'] = ((Newsletter.DRAFT, _('Default')),)
            return db_field.formfield(**kwargs)
        return super(NewsletterAdmin, self).formfield_for_choice_field(
            db_field, request, **kwargs)

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        if db_field.name == 'test_contacts':
            queryset = Contact.objects.filter(tester=True)
            if not request.user.is_superuser and USE_WORKGROUPS:
                contacts_pk = request_workgroups_contacts_pk(request)
                queryset = queryset.filter(pk__in=contacts_pk)
            kwargs['queryset'] = queryset
        return super(NewsletterAdmin, self).formfield_for_manytomany(
            db_field, request, **kwargs)

    def save_model(self, request, newsletter, form, change):
        workgroups = []
        if not newsletter.pk and not request.user.is_superuser \
               and USE_WORKGROUPS:
            workgroups = request_workgroups(request)

        if newsletter.content.startswith('http://'):
            if CAN_USE_PREMAILER:
                try:
                    premailer = Premailer(newsletter.content.strip())
                    newsletter.content = premailer.transform()
                except PremailerError:
                    self.message_user(request, _('Unable to download HTML, due to errors within.'))
            else:
                self.message_user(request, _('Please install lxml for parsing an URL.'))
        if not request.user.has_perm('newsletter.can_change_status'):
            newsletter.status = form.initial.get('status', Newsletter.DRAFT)

        newsletter.save()

        for workgroup in workgroups:
            workgroup.newsletters.add(newsletter)

    def historic_link(self, newsletter):
        """Display link for historic"""
        if newsletter.contactmailingstatus_set.count():
            return u'<a href="%s">%s</a>' % (newsletter.get_historic_url(), _('View historic'))
        return _('Not available')
    historic_link.allow_tags = True
    historic_link.short_description = _('Historic')

    def statistics_link(self, newsletter):
        """Display link for statistics"""
        if (
            newsletter.status == Newsletter.SENDING or
            newsletter.status == Newsletter.SENT
        ):
            return u"""
                <a href="%(href)s">
                    <img
                        src="%(url)sadmin/images/statistics.png"
                        alt="%(title)s" title="%(title)s"
                    />
                </a>""" % {
                    'href':     newsletter.get_statistics_url(),
                    'url':      settings.MEDIA_URL,
                    'title':    _('View statistics')
                }
        return _('Not available')
    statistics_link.allow_tags = True
    statistics_link.short_description = _('St')

    def send_mail_test(self, request, queryset):
        """Send newsletter in test"""
        for newsletter in queryset:
            if newsletter.test_contacts.count():
                mailer = Mailer(newsletter, test=True)
                try:
                    mailer.run()
                except HTMLParseError:
                    self.message_user(request, _('Unable send newsletter, due to errors within HTML.'))
                    continue
                self.message_user(request, _('%s succesfully sent.') % newsletter)
            else:
                self.message_user(request, _('No test contacts assigned for %s.') % newsletter)
    send_mail_test.short_description = _('Send test email')

    def make_ready_to_send(self, request, queryset):
        """Make newsletter ready to send"""
        queryset = queryset.filter(status=Newsletter.DRAFT)
        for newsletter in queryset:
            newsletter.status = Newsletter.WAITING
            newsletter.save()
        self.message_user(request, _('%s newletters are ready to send') % queryset.count())
    make_ready_to_send.short_description = _('Make ready to send')

    def make_cancel_sending(self, request, queryset):
        """Cancel the sending of newsletters"""
        queryset = queryset.filter(Q(status=Newsletter.WAITING) |
                                   Q(status=Newsletter.SENDING))
        for newsletter in queryset:
            newsletter.status = Newsletter.CANCELED
            newsletter.save()
        self.message_user(request, _('%s newletters are cancelled') % queryset.count())
    make_cancel_sending.short_description = _('Cancel the sending')

##############################
# CONTACT                    #
##############################
contacts_imported = Signal(providing_args=['source', 'type'])
class ContactExtraFieldsInline(admin.TabularInline):
    model = ContactExtraFields
    extra = 0
    fieldsets = ((None, {'fields': (('name', 'value'))}),)

class ContactAdmin(ExtendibleModelAdmin):
    model = Contact
    save_on_top = True
    list_display = (
        'email', 'tester', 'subscriber',
        'valid', 'total_subscriptions', 'creation_date',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    list_per_page = 30
    #list_filter = ('subscriber', 'valid', 'tester', 'creation_date', 'modification_date')
    search_fields = ('email',)
    fieldsets = (
        (None, {'fields': ('email',)}),
        (_('Status'), {'fields': ('subscriber', 'valid', 'tester')}),
      )
    actions = ['create_mailinglist', 'export_vcard', 'export_excel']
    inlines = [ContactExtraFieldsInline,]

    def queryset(self, request):
        queryset = super(ContactAdmin, self).queryset(request)
        if not request.user.is_superuser and USE_WORKGROUPS:
            contacts_pk = request_workgroups_contacts_pk(request)
            queryset = queryset.filter(pk__in=contacts_pk)
        return queryset

    def save_model(self, request, contact, form, change):
        workgroups = []
        if not contact.pk and not request.user.is_superuser \
               and USE_WORKGROUPS:
            workgroups = request_workgroups(request)
        contact.save()
        for workgroup in workgroups:
            workgroup.contacts.add(contact)

#    def related_object_admin(self, contact):
#        """Display link to related object's admin"""
#        if contact.content_type and contact.object_id:
#            admin_url = reverse(
#                'admin:%s_%s_change' % (
#                    contact.content_type.app_label,
#                    contact.content_type.model
#                ),
#                args=(contact.object_id,)
#            )
#            return '%s: <a href="%s">%s</a>' % (
#                contact.content_type.model.capitalize(),
#                admin_url,
#                contact.content_object.__unicode__()
#            )
#        return _('No relative object')
#    related_object_admin.allow_tags = True
#    related_object_admin.short_description = _('Related object')

    def total_subscriptions(self, contact):
        """Display user subscriptions to unsubscriptions"""
        subscriptions = contact.subscriptions().count()
        unsubscriptions = contact.unsubscriptions().count()
        return '%s / %s' % (subscriptions - unsubscriptions, subscriptions)
    total_subscriptions.short_description = _('Total subscriptions')

    def export_vcard(self, request, queryset, export_name=''):
        """Export selected contact in VCard"""
        return vcard_contacts_export_response(queryset)
    export_vcard.short_description = _('Export contacts as VCard')

    def export_excel(self, request, queryset, export_name=''):
        """Export selected contact in Excel"""
        if not export_name:
            export_name = 'contacts_edn_%s' % datetime.now().strftime('%d-%m-%Y')
        return ExcelResponse(queryset, export_name)
    export_excel.short_description = _('Export contacts in Excel')

    def create_mailinglist(self, request, queryset):
        """Create a mailing list from selected contact"""
        when = str(datetime.now()).split('.')[0]
        new_mailing = MailingList(name=_('New mailinglist at %s') % when,
                                  description=_('New mailing list created in admin at %s') % when)
        new_mailing.save()
        new_mailing.subscribers = queryset.all()

        if not request.user.is_superuser and USE_WORKGROUPS:
            for workgroup in request_workgroups(request):
                workgroup.mailinglists.add(new_mailing)

        self.message_user(request, _('%s succesfully created.') % new_mailing)
        return HttpResponseRedirect(reverse('admin:newsletter_mailinglist_change',
                                            args=[new_mailing.pk]))
    create_mailinglist.short_description = _('Create a mailinglist')

    def importation(self, request):
        """Import contacts from a VCard"""
        opts = self.model._meta

        if request.FILES:
            source = request.FILES.get('source')
            if not request.user.is_superuser and USE_WORKGROUPS:
                workgroups = request_workgroups(request)
            else:
                workgroups = []
            inserted = import_dispatcher(source, request.POST['type'],
                                         workgroups)
            if inserted:
                contacts_imported.send(sender=self, source=source,
                                       type=request.POST['type'])

            self.message_user(request, _('%s contacts succesfully imported.') % inserted)

        context = {'title': _('Contact importation'),
                   'opts': opts,
                   'root_path': self.admin_site.root_path,
                   'app_label': opts.app_label}

        return render_to_response('newsletter/contact_import.html',
                                  context, RequestContext(request))

    def filtered_request_queryset(self, request):
        """Return queryset filtered by the admin list view"""
        cl = ChangeList(request, self.model, self.list_display,
                        self.list_display_links, self.list_filter,
                        self.date_hierarchy, self.search_fields,
                        self.list_select_related, self.list_per_page,
                        self.list_editable, self)
        return cl.get_query_set()

    def creation_mailinglist(self, request):
        """Create a mailing list form the filtered contacts"""
        return self.create_mailinglist(request, self.filtered_request_queryset(request))

    def exportation_vcard(self, request):
        """Export filtered contacts in VCard"""
        return self.export_vcard(request, self.filtered_request_queryset(request),
                                 'contacts_edn_%s' % datetime.now().strftime('%d-%m-%Y'))

    def exportation_excel(self, request):
        """Export filtered contacts in Excel"""
        return self.export_excel(request, self.filtered_request_queryset(request),
                                 'contacts_edn_%s' % datetime.now().strftime('%d-%m-%Y'))

    def get_urls(self):
        urls = super(ContactAdmin, self).get_urls()
        my_urls = patterns('',
            url(r'^import/$',
                self.admin_site.admin_view(self.importation),
                name='newsletter_contact_import'),
            url(r'^create_mailinglist/$',
                self.admin_site.admin_view(self.creation_mailinglist),
                name='newsletter_contact_create_mailinglist'),
            url(r'^export_vcard/$',
                self.admin_site.admin_view(self.exportation_vcard),
                name='newsletter_contact_export_vcard'),
            url(r'^export_excel/$',
                self.admin_site.admin_view(self.exportation_excel),
                name='newsletter_contact_export_excel'),
        )
        return my_urls + urls

##############################
# MAILING LIST               #
##############################
class MailingListAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("newsletter/css/admin_newsletter.css",)}

    date_hierarchy = 'creation_date'
    list_display = (
        'creation_date', 'name', 'description',
        'subscribers_count', 'unsubscribers_count',
        #'exportation_link',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    list_editable = ('name', 'description')
    #list_filter = ('creation_date', 'modification_date')
    #search_fields = ('name', 'description',)
    #filter_horizontal = ['subscribers', 'unsubscribers']
    fieldsets = (
        (None, {'fields': ('name', 'description',)}),
    #    (None, {'fields': ('subscribers',)}),
    #    (None, {'fields': ('unsubscribers',)}),
    )
    actions = ['merge_mailinglist']

    def queryset(self, request):
        queryset = super(MailingListAdmin, self).queryset(request)
        if not request.user.is_superuser and USE_WORKGROUPS:
            mailinglists_pk = request_workgroups_mailinglists_pk(request)
            queryset = queryset.filter(pk__in=mailinglists_pk)
        return queryset

    def save_model(self, request, mailinglist, form, change):
        workgroups = []
        if not mailinglist.pk and not request.user.is_superuser \
               and USE_WORKGROUPS:
            workgroups = request_workgroups(request)
        mailinglist.save()
        for workgroup in workgroups:
            workgroup.mailinglists.add(mailinglist)

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        if 'subscribers' in db_field.name and not request.user.is_superuser \
               and USE_WORKGROUPS:
            contacts_pk = request_workgroups_contacts_pk(request)
            kwargs['queryset'] = Contact.objects.filter(pk__in=contacts_pk)
        return super(MailingListAdmin, self).formfield_for_manytomany(
            db_field, request, **kwargs)

    def merge_mailinglist(self, request, queryset):
        """Merge multiple mailing list"""
        if queryset.count() == 1:
            self.message_user(request, _('Please select a least 2 mailing list.'))
            return None

        subscribers = {}
        unsubscribers = {}
        for ml in queryset:
            for contact in ml.subscribers.all():
                subscribers[contact] = ''
            for contact in ml.unsubscribers.all():
                unsubscribers[contact] = ''

        when = str(datetime.now()).split('.')[0]
        new_mailing = MailingList(name=_('Merging list at %s') % when,
                                  description=_('Mailing list created by merging at %s') % when)
        new_mailing.save()
        new_mailing.subscribers = subscribers.keys()
        new_mailing.unsubscribers = unsubscribers.keys()

        if not request.user.is_superuser and USE_WORKGROUPS:
            for workgroup in request_workgroups(request):
                workgroup.mailinglists.add(new_mailing)

        self.message_user(request, _('%s succesfully created by merging.') % new_mailing)
        return HttpResponseRedirect(reverse('admin:newsletter_mailinglist_change',
                                            args=[new_mailing.pk]))
    merge_mailinglist.short_description = _('Merge selected mailinglists')

    def exportation_link(self, mailinglist):
        ## FIX vcard.py - manca first_name e last_name
        """Display link for exportation"""
        return u'<a href="%s">%s</a>' % (
            reverse(
                'admin:newsletter_mailinglist_export',
                args=[mailinglist.pk]
            ),
            _('Export Subscribers')
        )
    exportation_link.allow_tags = True
    exportation_link.short_description = _('Export')

    def export_subscribers(self, request, mailinglist_id):
        """Export subscribers in the mailing in VCard"""
        mailinglist = get_object_or_404(MailingList, pk=mailinglist_id)
        name = 'contacts_%s' % mailinglist.name
        return vcard_contacts_export_response(mailinglist.subscribers.all(), name)

    ### FIX vcard.py - manca first_name e last_name
    #def get_urls(self):
    #    urls = super(MailingListAdmin, self).get_urls()
    #    my_urls = patterns('',
    #        url(
    #            r'^export/(?P<mailinglist_id>\d+)/$',
    #            self.admin_site.admin_view(self.export_subscribers),
    #            name='newsletter_mailinglist_export'
    #        ),
    #    )
    #    return my_urls + urls

class CreditAdminInline(admin.TabularInline):
    model = Credit
    extra = 0
    fieldsets = ((None, {'fields': (('dow_from', 'hour_from', 'dow_to', 'hour_to', 'mails_hour'))}),)

##############################
# BOUNCER                    #
##############################
class BounceAccountAdminForm(forms.ModelForm):
    def __init__(self, *args, **kws):
        super(BounceAccountAdminForm, self).__init__(*args, **kws)
        self.fields['password'] = forms.CharField(widget=PasswordInput(), required = False)

    class Meta:
        model = BounceAccount

class BounceAccountAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("newsletter/css/admin_newsletter.css",)}

    form = BounceAccountAdminForm
    list_display = (
        'email', 'host', 'port', 'user', 'tls',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    fieldsets = (
        (None, {'fields': ('email', )}),
        (_('Configuration'), {'fields': (
            'host', 'port', 'user', 'password', 'tls',
        )}),
    )
##############################
# SMTP                       #
##############################
class SMTPServerAdminForm(forms.ModelForm):
    def __init__(self, *args, **kws):
        super(SMTPServerAdminForm, self).__init__(*args, **kws)
        self.fields['password'] = forms.CharField(widget=PasswordInput(), required = False)

    def clean_headers(self):
        """Check if the headers are well formated"""
        for line in self.cleaned_data['headers'].splitlines():
            elems = line.split(':')
            if len(elems) < 2:
                raise ValidationError(_('Invalid syntax, do not forget the ":".'))
            if len(elems) > 2:
                raise ValidationError(_('Invalid syntax, several assignments by line.'))

        return self.cleaned_data['headers']

    class Meta:
        model = SMTPServer

class SMTPServerAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("newsletter/css/admin_newsletter.css",)}

    form = SMTPServerAdminForm
    list_display = (
        'name', 'host', 'port', 'user', 'tls', 'get_credits',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    #list_filter = ('tls',)
    #search_fields = ('name', 'host', 'user')
    fieldsets = (
        (None, {'fields': ('name', )}),
        (_('Configuration'), {'fields': (
            'host', 'port', 'user', 'password', 'tls', 'mails_hour',
        )}),
        (_('Miscellaneous'), {
            'fields': ('headers',),
            'classes': ('collapse', )
        }),
    )
    actions = ['check_connections']
    inlines = [CreditAdminInline,]

    def check_connections(self, request, queryset):
        """Check the SMTP connection"""
        message = '%s connection %s'
        for server in queryset:
            try:
                smtp = server.connect()
                if smtp:
                    status = 'OK'
                    smtp.quit()
                else:
                    status = 'KO'
            except:
                status = 'KO'
            self.message_user(request, message % (server.__unicode__(), status))
    check_connections.short_description = _('Check connection')

#    def get_urls(self):
#        from django.conf.urls.defaults import patterns, url
#        urls = super(SMTPServerAdmin, self).get_urls()
#        my_urls = patterns('',
#            url(
#                r'^(?P<smtp_id>\d+)/set_credits/$',
#                self.admin_site.admin_view(newsletter_views.set_credits),
#                name='newsletter_smtpserver_set_credits'
#            ),
#        )
#        return my_urls + urls

#    def set_credits(self, obj):
#        return """
#            <a class="iframe" href="%(url_reverse)s?_popup=1&nobutton=1">
#                <img src='%(media_url)sadmin/images/comment.png' title=\"Set Credits\" style=\"vertical-align:middle;\" />
#            </a>
#        """ % {
#            'url_reverse': reverse('admin:newsletter_smtpserver_set_credits', args=str(obj.id)),
#            'media_url': settings.MEDIA_URL,
#        }
#    set_credits.allow_tags=True


##############################
# Link                       #
##############################
class LinkAdmin(ExtendibleModelAdmin):
    list_display = (
        'title', 'url', 'creation_date',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )

##############################
# Status                     #
##############################
class ContactMailingStatusSearchForm(forms.ModelForm):
    class Meta:
        model = ContactMailingStatus
        display_fields = (
            ('status', 'rfc_status__icontains', 'diagnostic_code__icontains'),
            ('creation_date__gte', 'creation_date__lte',),
            ('first_open_date__gte', 'first_open_date__lte',),
            ('newsletter', 'opened_newsletter',),
            'contact__email__icontains',
            'link__title__icontains'
        )

    STATUS = ContactMailingStatus.STATUS_CHOICES
    STATUS.insert(0, ('', '---'))
    OPENED = ((None, "---"), (1, _("yes")),(0, _("no")),)

    def __init__(self, *args, **kws):
        super(ContactMailingStatusSearchForm, self).__init__(*args, **kws)
        self.fields['status'] = forms.ChoiceField(label=_("Status"), choices=self.STATUS, required=False)
        self.fields['creation_date__gte'] = JumboDateField(label=_("Sent Date from"), required=False)
        self.fields['creation_date__lte'] = JumboDateField(label=_("Sent Date to"), required=False)
        self.fields['first_open_date__gte'] = JumboDateField(label=_("First Open Date from"), required=False)
        self.fields['first_open_date__lte'] = JumboDateField(label=_("First Open Date to"), required=False)
        self.fields['contact__email__icontains'] = forms.CharField(label=_("Contact Email icontains"), required=False)
        self.fields['link__title__icontains'] = forms.CharField(label=_("Link Title icontains"), required=False)
        self.fields['rfc_status__icontains'] = forms.CharField(label=_("RFC Status icontains"), required=False)
        self.fields['diagnostic_code__icontains'] = forms.CharField(label=_("Diagnostic Code icontains"), required=False)
        self.fields['opened_newsletter'] = forms.ChoiceField(label=_("Opened"), choices=self.OPENED, required=False)

    def change_search_filter(self, lookup_params, qs):
        # data invio
        if lookup_params.has_key('creation_date__lte'):
            date = lookup_params['creation_date__lte']
            try:
                fixed_datetime = datetime(date.year, date.month, date.day, 23, 59, 59)
                qs = qs.filter(creation_date__lte = fixed_datetime)
            except Exception, e:
                raise IncorrectLookupParameters
            del lookup_params['creation_date__lte']

        # data prima apertura
        if lookup_params.has_key('first_open_date__lte'):
            date = lookup_params['first_open_date__lte']
            try:
                fixed_datetime = datetime(date.year, date.month, date.day, 23, 59, 59)
                qs = qs.filter(first_open_date__lte = fixed_datetime)
            except Exception, e:
                raise IncorrectLookupParameters
            del lookup_params['first_open_date__lte']

        # opened
        if lookup_params.has_key('opened_newsletter'):
            opened = lookup_params['opened_newsletter']
            if opened == "0":
                qs = qs.filter(
                    opened = 0, opened_on_site = 0, link_opened = 0
                )
            if opened == "1":
                qs = qs.filter(
                    Q(opened__gte = 1) |
                    Q(opened_on_site__gte = 1) |
                    Q(link_opened__gte = 1)
                )
            del lookup_params['opened_newsletter']

        return lookup_params, qs
        

class ContactMailingStatusAdmin(ExtendibleModelAdmin):
    search_form = ContactMailingStatusSearchForm
    list_display = (
        'get_newsletter', 'get_status', 'creation_date', 'get_contact', 'first_open_date', 'opened',
        'opened_on_site', 'link_opened', 'unsubscription'
    )
    default_group_by = 'newsletter'

    def get_newsletter(self, result):
        return result.newsletter.title
    get_newsletter.short_description = _("Newsletter")

    def get_contact(self, result):
        return result.contact.email
    get_contact.short_description = _("Contact")

    def get_status(self, result):
        if result.rfc_status:
            field = """
                <span title="%s: %s">%s</span>
            """ % (
                result.rfc_status,
                result.diagnostic_code or "",
                result.get_status_display()
            )
        else:
            field = """
                <span title="%s">%s</span>
            """ % (
                result.diagnostic_code or "",
                result.get_status_display()
            )
        return field        
    get_status.short_description = _("Status")
    get_status.allow_tags = True

class LinkContactMailingStatusAdmin(ExtendibleModelAdmin):
    list_display = ('get_contact_mailing_status', 'get_link', 'click', 'date_create')

    def get_contact_mailing_status(self, result):
        return "%s -%s" % (result.contact_mailing_status.newsletter, result.contact_mailing_status.contact)
    get_contact_mailing_status.short_description = _("Contact MS")

    def get_link(self, result):
        return result.link.url
    get_link.short_description = _("Link")


admin.site.register(Newsletter, NewsletterAdmin)
admin.site.register(Contact, ContactAdmin)
admin.site.register(MailingList, MailingListAdmin)
admin.site.register(SMTPServer, SMTPServerAdmin)
admin.site.register(BounceAccount, BounceAccountAdmin)
#admin.site.register(Credit, CreditAdmin)
admin.site.register(Link, LinkAdmin)
admin.site.register(ContactMailingStatus, ContactMailingStatusAdmin)
admin.site.register(LinkContactMailingStatus, LinkContactMailingStatusAdmin)
admin.site.register(WorkGroup)
