"""
Parse a DSN message and gather information needed to decide what to do with the
original recipient
"""


import re
import email

class NotDSNError(Exception): pass

class DSNParser(object):

    """A message parser for the DSN message retrieved via pop3

    Delivery Notification Status is defined in rfc3464_. In chapter 2
    a,b,c,d it defines that a message is of type multipart/report with 2 or
    3 subparts. The 2^nd one is the one used for automatic parsing and is
    subdiveded in subparts that are parsed by this class
    
    This parser sets attributes

    .. _rfc3464: http://www.ietf.org/rfc/rfc3464.txt
    
    """

    email = None
    """The email.message instance of the original message"""

    original_recipient = None
    "The original recipient of the message"

    final_recipient = None
    "The final recipient of the message (e.g. the original is expanded as alias)"

    status = None
    "The status notified in this DSN: 5.1.1, 5.2.0 are example values"

    diagnostic_code = None
    "The Diagnostic-Code header field. May be usefull if other codes are criptical"


    notification = None
    "The uman readable notification message (1^st part of the dns)" 
    
    update_contact = None
    "Boolean. If True the contact must be updated"

    def __init__(self, txt=None, filename=None, msg=None):
        if txt:
            self.email = email.message_from_string(txt)
        elif filename:
            self.email = email.message_from_file(filename)
        elif msg:
            self.email = msg

        if not self.email.is_multipart():
            raise NotDSNError("Message is not a DSN")
            
        self.parse_message()
        self.learn_from_state()

    def parse_message(self):
        # Response:
        # http://www.ietf.org/rfc/rfc3464.txt  Format of Delivery status notification
        # http://tools.ietf.org/html/rfc3834
        # Return codes:
        # http://www.ietf.org/rfc/rfc1891.txt
        # http://en.wikipedia.org/wiki/Non_delivery_report

        # the DSN mesage is multipart of type multipart/report
        # the 1^st part contains a human readable description
        # the 2^nd part is of content-type message/delivery-status

        for msg in self.email.get_payload():
            if msg.has_key('Content-Description'):

                # 1st subpart of multipart/report
                if msg['Content-Description'] == 'Notification':
                    self.notification = msg.get_payload()
                # 2nd subpart. Suitable for automatic parsing
                elif re.search('message/delivery-status', msg['Content-Type']):

                    # 1st sub-sub-part is for reporting MTA
                    # 2nd+ sub-sub-part is for each final-recipient
                    for m in msg.get_payload():
                        if m['Status']:
                            self.status = m['Status']
                        if m['Diagnostic-Code']:
                            self.diagnostic_code = m['Diagnostic-Code']
                        if m['Original-Recipient']:
                            addr_type, address = m['Original-Recipient'].split(';')
                            self.original_recipient = address.strip()
                        if m['Final-Recipient']:
                            addr_type, address = m['Final-Recipient'].split(';')
                            self.final_recipient = address.strip()
                            
                ## 3rd part. If resent holds list-id
                elif re.search('Undelivered Message', msg['Content-Description']):
                    # may still be  Undelivered Message Headers or  Undelivered Message 
                    if msg.is_multipart():
                        m = msg.get_payload()[0]
                    else:
                        m = email.message_from_string(msg.get_payload())
                    if m['List-ID']:
                        list_id = m['List-ID']
                        self.list_id = re.sub('[<>]', '', list_id)

    def learn_from_state(self):
        """Read the internal state and determine if an update of the
        Contact info is needed
        """
        # RFC 1894 dictates what status eans at chapter 2.3.4
        # The first sub-field indicates whether the delivery attempt was
        # successful (2 = success, 4 = persistent temporary failure, 5 =
        # permanent failure). 
    
        if self.status.split('.')[0] == '5':
            self.update_contact = True
        else:
            self.update_contact = False
            
        # In any case don't udpate a user if the final recipient is not the original
        # It's possible that an alias expanded in such a way that a part has arrived
        
        if not self.final_recipient == self.original_recipient:
            self.update_contact = False

        
    def __repr__(self):
        return "<DSN Message: %s, %s (%s)>" % (self.status, self.original_recipient, self.list_id)
