"""Models for emencia.django.newsletter"""
from smtplib import SMTP
from smtplib import SMTPHeloError
from datetime import datetime, time
from datetime import timedelta
from picklefield.fields import PickledObjectField

from django.db import models
from django.db.models import Q
from django.utils.encoding import smart_str
from django.db import connection, transaction
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import Group
from django.utils.encoding import force_unicode


from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel, HTMLField
from jumbo.db.models import Settings as JSettings
from jumbo.utils.functions import get_settings

#from tagging.fields import TagField

from newsletter.managers import ContactManager
from newsletter.settings import MAILER_HARD_LIMIT
from newsletter.settings import NEWSLETTER_BASE_PATH
from newsletter.utils.vcard import vcard_contact_export

# Patch for Python < 2.6
try:
    getattr(SMTP, 'ehlo_or_helo_if_needed')
except AttributeError:
    def ehlo_or_helo_if_needed(self):
        if self.helo_resp is None and self.ehlo_resp is None:
            if not (200 <= self.ehlo()[0] <= 299):
                (code, resp) = self.helo()
                if not (200 <= code <= 299):
                    raise SMTPHeloError(code, resp)
    SMTP.ehlo_or_helo_if_needed = ehlo_or_helo_if_needed

class BounceAccount(models.Model):
    """Bounce Account"""
    email = models.EmailField(_('email'),)
    host = models.CharField(_('server host'), max_length=255)
    user = models.CharField(_('server user'), max_length=128, blank=True,
                            help_text=_('Leave it empty if the host is public.'))
    password = models.CharField(_('server password'), max_length=128, blank=True,
                                help_text=_('Leave it empty if the host is public.'))
    port = models.IntegerField(_('server port'), default=110)
    tls = models.BooleanField(_('server use TLS'))

    def __unicode__(self):
        return '%s (%s)' % (self.email, self.host)

    class Meta:
        verbose_name = _('Bounce Account')
        verbose_name_plural = _('Bounce Accounts')

class SMTPServer(models.Model):
    """Configuration of a SMTP server"""
    name = models.CharField(_('name'), max_length=255)
    host = models.CharField(_('server host'), max_length=255)
    user = models.CharField(_('server user'), max_length=128, blank=True,
                            help_text=_('Leave it empty if the host is public.'))
    password = models.CharField(_('server password'), max_length=128, blank=True,
                                help_text=_('Leave it empty if the host is public.'))
    port = models.IntegerField(_('server port'), default=25)
    tls = models.BooleanField(_('server use TLS'))

    headers = models.TextField(_('custom headers'), blank=True,
                               help_text=_('key1: value1 key2: value2, splitted by return line.\n'\
                                           'Useful for passing some tracking headers if your provider allows it.'))
    mails_hour = models.IntegerField(_('mails per hour'), default=0)

    def connect(self):
        """Connect the SMTP Server"""
        smtp = SMTP(smart_str(self.host), int(self.port))
        smtp.ehlo_or_helo_if_needed()
        if self.tls:
            smtp.starttls()
            smtp.ehlo_or_helo_if_needed()

        if self.user or self.password:
            smtp.login(smart_str(self.user), smart_str(self.password))
        return smtp

    def get_credits(self):
        """Return how many mails the server can send"""
        now = datetime.today()
        last_hour = datetime(now.year, now.month, now.day, now.hour, 0, 0, 0)

        sent_curr_hour = ContactMailingStatus.objects.filter(
            models.Q(status=ContactMailingStatus.SENT) |
            models.Q(status=ContactMailingStatus.SENT_TEST) |
            models.Q(status=ContactMailingStatus.BOUNCED),
            newsletter__server=self,
            creation_date__gte=last_hour).count()

        time_now = time(now.hour, now.minute)
        # Cerco se ci sono delle eccezioni ai credits
        exception_credits = self.credits.filter(dow_from__lte=now.weekday(),
                                               hour_from__lte=time_now,
                                               dow_to__gte=now.weekday(),
                                               hour_to__gte=time_now)

        if exception_credits:
            mails_hour = exception_credits[0].mails_hour
        else:
            mails_hour = self.mails_hour

        if not mails_hour or mails_hour > MAILER_HARD_LIMIT:
            return MAILER_HARD_LIMIT

        return mails_hour - sent_curr_hour

    @property
    def custom_headers(self):
        if self.headers:
            headers = {}
            for header in self.headers.splitlines():
                if header:
                    key, value = header.split(':')
                    headers[key.strip()] = value.strip()
            return headers
        return {}

    def __unicode__(self):
        return '%s (%s)' % (self.name, self.host)

    class Meta:
        verbose_name = _('SMTP server')
        verbose_name_plural = _('SMTP servers')


DOW_CHOICES = (
    (0, _('Monday')),
    (1, _('Tuesday')),
    (2, _('Wendesday')),
    (3, _('Thursday')),
    (4, _('Friday')),
    (5, _('Saturday')),
    (6, _('Sunday')),
)
HOUR_CHOICES = []
for i in range(24):
    HOUR_CHOICES.append((time(i), '%s:00' % i))

#HOUR_CHOICES = (
#    (time(0), '00:00'), (time(0,30), '00:30'), (time(1), '01:00'), (time(1,30), '01:30'), (time(2), '02:00'),
#    (time(2,30), '02:30'), (time(3), '03:00'), (time(3,30), '03:30'), (time(4), '04:00'), (time(4,30), '04:30'),
#    (time(5), '05:00'), (time(5,30), '05:30'), (time(6), '06:00'), (time(6,30), '06:30'), (time(7), '07:00'),
#    (time(7,30), '07:30'), (time(8), '08:00'), (time(8,30), '08:30'), (time(9), '09:00'), (time(9,30), '09:30'),
#    (time(10), '10:00'), (time(10,30), '10:30'), (time(11), '11:00'), (time(11,30), '11:30'), (time(12), '12:00'),
#    (time(12,30), '12:30'), (time(13), '13:00'), (time(13,30), '13:30'), (time(14), '14:00'), (time(14,30), '14:30'),
#    (time(15), '15:00'), (time(15,30), '15:30'), (time(16,00), '16:00'), (time(16,30), '16:30'), (time(17), '17:00'),
#    (time(17,30), '17:30'), (time(18), '18:00'), (time(18,30), '18:30'), (time(19), '19:00'), (time(19,30), '19:30'),
#    (time(20), '20:00'), (time(20,30), '20:30'), (time(21,00), '21:00'), (time(21,30), '21:30'), (time(22), '22:00'),
#    (time(22,30), '22:30'), (time(23), '23:00'), (time(23,30), '23:30'),
#)
class Credit(models.Model):
    """How many mails server can send in a particular day at particular time"""
    smtp = models.ForeignKey(SMTPServer, verbose_name=_("smtp server"), related_name='credits')
    dow_from = models.IntegerField(choices=DOW_CHOICES, max_length=1)
    hour_from = models.TimeField(choices=HOUR_CHOICES)
    dow_to = models.IntegerField(choices=DOW_CHOICES, max_length=1)
    hour_to = models.TimeField(choices=HOUR_CHOICES)
    #day_of_week = models.IntegerField(_("day"),)
    #hour = models.IntegerField(_("hour"))
    mails_hour = models.IntegerField(_('mails per hour'), default=0)

    def __unicode__(self):
        return u"%s" % self.mails_hour

    class Meta:
        verbose_name = _('Credit')
        verbose_name_plural = _('Credits')
        unique_together = ("smtp", "dow_from", "hour_from", "dow_to", "hour_to")
        ordering = ("smtp", "dow_from", "hour_from")

class ContactExtraFields(models.Model):
    contact = models.ForeignKey("Contact", related_name = 'contact_extra_fields')
    name = models.CharField(max_length=100)
    value = models.CharField(max_length=255)

class Contact(models.Model):
    """Contact for emailing"""
    ## collegamento a qualsiasi modello
    #content_type = models.ForeignKey(ContentType)
    #object_id = models.PositiveIntegerField()
    #content_object = generic.GenericForeignKey('content_type', 'object_id')

    email = models.EmailField(_('email'), unique=True)
    subscriber = models.BooleanField(_('subscriber'), default=True)
    valid = models.BooleanField(_('valid email'), default=True)
    tester = models.BooleanField(_('contact tester'), default=False)
    #pickle_args = PickledObjectField(_('additional arguments'), default=True)
    #tags = TagField(_('tags'))

    creation_date = models.DateTimeField(_('creation date'), auto_now_add=True)
    modification_date = models.DateTimeField(_('modification date'), auto_now=True)

    extra_fields = models.ForeignKey(ContactExtraFields, related_name="contacts",
                                     blank = True, null = True)

    objects = ContactManager()

    def subscriptions(self):
        """Return the user subscriptions"""
        return MailingList.objects.filter(subscribers=self)

    def unsubscriptions(self):
        """Return the user unsubscriptions"""
        return MailingList.objects.filter(unsubscribers=self)

    def vcard_format(self):
        return vcard_contact_export(self)

    def mail_format(self):
        #if self.first_name and self.last_name:
        #    return '%s %s <%s>' % (self.last_name, self.first_name, self.email)
        return self.email
    mail_format.short_description = _('mail format')

    def get_extra_fields(self):
        d_extra_fields = {}
        for f in self.contact_extra_fields.all():
            d_extra_fields[f.name] = f.value
        return d_extra_fields
    extra = property(get_extra_fields)

    def get_absolute_url(self):
        if self.content_type and self.object_id:
            return self.content_object.get_absolute_url()
        return reverse('admin:newsletter_contact_change', args=(self.pk,))

    def __unicode__(self):
        return self.email

    class Meta:
        ordering = ('creation_date',)
        verbose_name = _('subscripter')
        verbose_name_plural = _('subscripters')


class MailingList(models.Model):
    """Mailing list"""
    name = models.CharField(_('name'), max_length=255)
    description = models.TextField(_('description'), blank=True)

    subscribers = models.ManyToManyField(
        Contact,
        verbose_name=_('subscribers'),
        related_name='mailinglist_subscriber',
    )
    unsubscribers = models.ManyToManyField(
        Contact,
        verbose_name=_('unsubscribers'),
        related_name='mailinglist_unsubscriber',
        null=True, blank=True
    )

    creation_date = models.DateTimeField(_('creation date'), auto_now_add=True)
    modification_date = models.DateTimeField(_('modification date'), auto_now=True)

    def clone(self):
        oggi = datetime.today()
        new_obj = MailingList()
        new_obj.name = '%s_%s' % (self.name, oggi.strftime('%d-%m-%Y'))
        new_obj.description = self.description
        new_obj.save()

        return new_obj

    def subscribers_count(self):
        return self.subscribers.all().count()
    subscribers_count.short_description = _('subscribers')

    def unsubscribers_count(self):
        return self.unsubscribers.all().count()
    unsubscribers_count.short_description = _('unsubscribers')

    def expedition_set(self):
        """
        restituisce i sottoscrittori di una mailinglist
        """
        ## lentissima
        # unsubscribers_id = self.unsubscribers.values_list('id', flat=True)
        # return self.subscribers.filter(valid=True).exclude(
        #     id__in=unsubscribers_id).order_by('id')
        return self.subscribers.filter(valid=True).exclude(mailinglist_unsubscriber=self).order_by('id')

    def add_total_subscribers(self, list_subscribers):
        self.subscribers.clear()
        for subscriber in list_subscribers:
            self.subscribers.add(subscriber)
        self.save()

    def add_subscriber(self, email, ordering=0, extra_fields={}):
        if not email:
            return 2 # Email non presente
        if self.subscribers.filter(email=email):
            return 1 # Sottoscrittore gia' presente

        contact = Contact.objects.get_or_create(email = email)[0]
        self.subscribers.add(contact) # = MailingListOrderedSubscribers(mailing_list=self, subscriber=contact, ordering=ordering)
        #ordered_subscriber.save()

        for key in extra_fields.keys():
            cef, created = ContactExtraFields.objects.get_or_create(contact=contact,
                                                                    name = key)
            cef.value = extra_fields[key]
            cef.save()

        return 0

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ('-creation_date',)
        verbose_name = _('mailing list')
        verbose_name_plural = _('mailing lists')

class Newsletter(models.Model):
    """Newsletter to be sended to contacts"""
    DRAFT = 0
    WAITING = 1
    SENDING = 2
    SENT = 4
    CANCELED = 5

    STATUS_CHOICES = (
        (DRAFT, _('draft')),
        (WAITING, _('waiting sending')),
        (SENDING, _('sending')),
        (SENT, _('sent')),
        (CANCELED, _('canceled')),
    )

    title = models.CharField(
        _('title'),
        max_length=255,
        help_text=_("""
            You can use the "{{ UNIQUE_KEY }}" variable
            for unique identifier within the newsletter's title.
        """)
    )
    content = HTMLField(
        _('content'),
        help_text=_('write HTML code or paste an URL.'),
    )
    mailing_list = models.ForeignKey(
        MailingList,
        related_name = 'newsletter',
        verbose_name=_('mailing list')
    )
    test_contacts = models.ManyToManyField(
        Contact,
        verbose_name=_('test contacts'),
        blank=True, null=True
    )
    server = models.ForeignKey(
        SMTPServer,
        verbose_name=_('smtp server'),
    )
    bouncer =models.ForeignKey(
        BounceAccount,
        related_name='newsletters',
        verbose_name=_('bouncer'),
    )
    header_sender = models.CharField(
        _('sender'),
        max_length=255,
        default=get_settings("newsletter.settings", "NEWSLETTER_DEFAULT_HEADER_SENDER")
    )
    header_reply = models.CharField(
        _('reply to'),
        max_length=255,
        default=get_settings("newsletter.settings", "NEWSLETTER_DEFAULT_HEADER_REPLY")
    )
    status = models.IntegerField(
        _('status'),
        choices=STATUS_CHOICES,
        default=DRAFT
    )
    sending_date = models.DateTimeField(
        _('sending date'),
        default=datetime.now
    )
    slug = models.SlugField(
        help_text=_('Used for displaying the newsletter on the site.'),
        unique=True
    )
    creation_date = models.DateTimeField(
        _('creation date'),
        auto_now_add=True
    )
    modification_date = models.DateTimeField(
        _('modification date'),
        auto_now=True
    )

    def clone(self):
        oggi = datetime.today()
        new_obj = Newsletter()
        new_obj.title = '%s_%s' % (self.title, oggi.strftime('%d-%m-%Y'))
        new_obj.slug = new_obj.title
        new_obj.content = self.content
        new_obj.mailing_list = self.mailing_list
        new_obj.server = self.server
        new_obj.header_sender = self.header_sender
        new_obj.header_reply= self.header_reply
        new_obj.save()

        return new_obj

    def get_totals(self, date_stat_begin=None, date_stat_end=None):
        where_date = group_by = date_c = ""
        date_row = None
        if date_stat_begin:
            where_date += " AND creation_date>='%s-%s-%s 00:00:00'" % (
                date_stat_begin.year, date_stat_begin.month, date_stat_begin.day
            )
        if date_stat_end:
            where_date += " AND creation_date<='%s-%s-%s 23:59:59'" % (
                date_stat_end.year, date_stat_end.month, date_stat_end.day
            )
            group_by = " GROUP BY date(creation_date)"
            date_c = ", date(creation_date) AS date"
        
        cursor = connection.cursor()
        query = """
            SELECT
                count(*) AS mails,
                SUM(CASE
                    WHEN status = 0 THEN 1
                    ELSE 0
                END) AS sent,
                SUM(CASE
                    WHEN status = -1 THEN 1
                    ELSE 0
                END) AS sent_test,
                SUM(CASE
                    WHEN status = 1 THEN 1
                    ELSE 0
                END) AS error,
                SUM(CASE
                    WHEN status = 2 THEN 1
                    ELSE 0
                END) AS invalid,
                SUM(CASE
                    WHEN status = 9 THEN 1
                    ELSE 0
                END) AS bounced,
                SUM(opened) AS opened_on_client,
                SUM(CASE 
                    WHEN opened > 0 THEN 1
                    ELSE 0
                    END) AS unique_opened_on_client,
                SUM(opened_on_site) AS opened_on_site,
                SUM(CASE
                    WHEN opened_on_site > 0 THEN 1
                    ELSE 0
                END) AS unique_opened_on_site,
                SUM(link_opened) AS link_opened,
                SUM(CASE
                    WHEN link_opened > 0 THEN 1
                    ELSE 0
                END) AS unique_link_opened,
                SUM(opened + opened_on_site) AS tot_opened,
                SUM(CASE
                    WHEN opened > 0 OR opened_on_site > 0 THEN 1
                    ELSE 0
                END) AS unique_tot_opened
                %s
            FROM
                newsletter_contactmailingstatus
            WHERE
                newsletter_id = %s
                %s %s
        """ % (date_c, self.id, where_date, group_by)
        cursor.execute(query)
        rows = cursor.fetchall()
        totals = []

        for row in rows:
            if date_c:
                date_row = row[14]

            if int(row[0]) > 0:
                totals.append({
                    'mails':                            row[0],
                    'sent':                             row[1],
                    'sent_percent':                     row[1]*100.00/row[0],
                    'sent_test':                        row[2],
                    'sent_test_percent':                row[2]*100.00/row[0],
                    'error':                            row[3],
                    'error_percent':                    row[3]*100.00/row[0],
                    'invalid':                          row[4],
                    'invalid_percent':                  row[4]*100.00/row[0],
                    'bounced':                          row[5],
                    'bounced_percent':                  row[5]*100.00/row[0],
                    'opened_on_client':                 row[6],
                    'opened_on_client_percent':         row[6]*100.00/row[0],
                    'unique_opened_on_client':          row[7],
                    'unique_opened_on_client_percent':  row[7]*100.00/row[0],
                    'opened_on_site':                   row[8],
                    'opened_on_site_percent':           row[8]*100.00/row[0],
                    'unique_opened_on_site':            row[9],
                    'unique_opened_on_site_percent':    row[9]*100.00/row[0],
                    'link_opened':                      row[10],
                    'link_opened_percent':              row[10]*100.00/row[0],
                    'unique_link_opened':               row[11],
                    'unique_link_opened_percent':       row[11]*100.00/row[0],
                    'tot_opened':                       row[12],
                    'tot_opened_percent':               row[12]*100.00/row[0],
                    'unique_tot_opened':                row[13],
                    'unique_tot_opened_percent':        row[13]*100.00/row[0],
                    'tot_not_opened':                   row[1]-row[13],
                    'tot_not_opened_percent':           (row[1]-row[13])*100.00/row[0],
                    'date_c':                           date_row
                })
        return totals

    def get_newsletter_top_links(self, date_stat_begin=None, date_stat_end=None):
        """Return the most clicked links"""
        cursor = connection.cursor()
        query = """
            SELECT
                lcms.link_id,
                l.title,
                l.url,
                SUM (lcms.click) AS tot_clicks,
                SUM(CASE 
                    WHEN lcms.click > 0 THEN 1
                    ELSE 0
                    END) AS unique_clicks
            FROM
                newsletter_linkcontactmailingstatus AS lcms
                JOIN newsletter_contactmailingstatus AS cms ON cms.id=lcms.contact_mailing_status_id
                JOIN newsletter_link AS l ON l.id=lcms.link_id
            WHERE cms.newsletter_id=%s
            GROUP BY lcms.link_id, l.title, l.url
            ORDER BY tot_clicks DESC
        """ % (self.id)
        cursor.execute(query)
        rows = cursor.fetchall()
        links = []
        for row in rows:
            if row[0] != "":
                links.append({
                    'link_id':          row[0],
                    'title':            row[1],
                    'url':              row[2],
                    'tot_clicks':       row[3],                                
                    'unique_clicks':    row[4]                                
                })
        return links
             
    def subscriptions(self):
        return self.mailing_list.subscribers.count()
    subscriptions = property(subscriptions)

    def unsubscriptions(self):
        return self.mailing_status.filter(unsubscription=True).count()
    unsubscriptions = property(unsubscriptions)

    def unsubscriptions_perc(self):
        if self.subscriptions != 0:
            return float(self.unsubscriptions) / float(self.subscriptions) * 100
        return float(0)
    unsubscriptions_perc = property(unsubscriptions_perc)

    @models.permalink
    def get_absolute_url(self):
        return ('newsletter_newsletter_preview', (self.slug,))

    @models.permalink
    def get_historic_url(self):
        return ('newsletter_newsletter_historic', (self.slug,))

    @models.permalink
    def get_statistics_url(self):
        return ('newsletter_newsletter_statistics', (self.slug,))

    def __unicode__(self):
        return self.title

    class Meta:
        ordering = ('-creation_date',)
        verbose_name = _('newsletter')
        verbose_name_plural = _('newsletters')
        permissions = (
            ('can_change_status', 'Can change status'),
            ('view_newsletter',   'Can view newsletter'),
        )


class Link(models.Model):
    """Link sended in a newsletter"""
    title = models.CharField(_('title'), max_length=255)
    url = models.CharField(_('url'), max_length=255)

    creation_date = models.DateTimeField(_('creation date'), auto_now_add=True)

    def get_absolute_url(self):
        return self.url

    def __unicode__(self):
        return self.title

    class Meta:
        ordering = ('-creation_date',)
        verbose_name = _('link')
        verbose_name_plural = _('links')


class Attachment(models.Model):
    """Attachment file in a newsletter"""

    def get_newsletter_storage_path(self, filename):
        filename = force_unicode(filename)
        return '/'.join([NEWSLETTER_BASE_PATH, self.newsletter.slug, filename])

    newsletter = models.ForeignKey(Newsletter, verbose_name=_('newsletter'))
    title = models.CharField(_('title'), max_length=255)
    file_attachment = models.FileField(_('file to attach'), max_length=255,
                                       upload_to=get_newsletter_storage_path)

    class Meta:
        verbose_name = _('attachment')
        verbose_name_plural = _('attachments')

    def __unicode__(self):
        return self.title

    def get_absolute_url(self):
        return self.file_attachment.url


class ContactMailingStatus(models.Model):
    """Status of the reception"""
    SENT_TEST = -1
    SENT = 0
    ERROR = 1
    INVALID = 2
    #OPENED = 4
    #OPENED_ON_SITE = 5
    #LINK_OPENED = 6
    #UNSUBSCRIPTION = 7
    BOUNCED = 9

    STATUS_CHOICES = [
        (SENT_TEST, _('sent in test')),
        (SENT, _('sent')),
        (ERROR, _('error')),
        (INVALID, _('invalid email')),
        (BOUNCED, _('bounced email')),
    ]

    newsletter = models.ForeignKey(
        Newsletter,
        related_name = 'mailing_status',
        verbose_name=_('newsletter')
    )
    contact = models.ForeignKey(
        Contact,
        related_name = 'mailing_status',
        verbose_name=_('contact')
    )
    status = models.IntegerField(
        _('status'),
        choices=STATUS_CHOICES
    )
    opened = models.IntegerField(default=0)
    opened_on_site = models.IntegerField(default=0)
    link = models.ManyToManyField(
        Link,
        related_name = 'mailing_status',
        verbose_name=_('link'),
        blank=True, null=True,
        through = "LinkContactMailingStatus"
    )
    link_opened = models.IntegerField(default=0)
    unsubscription = models.BooleanField(default=0)
    rfc_status = models.CharField(
        max_length=10,
        blank=True, null=True
    )
    diagnostic_code = models.TextField(
        blank=True, null=True
    )

    creation_date = models.DateTimeField(_('creation date'), auto_now_add=True)
    first_open_date = models.DateTimeField(_('first open date'), blank=True, null=True)
    unsubscription_date = models.DateTimeField(_('unsubscription date'), blank=True, null=True)

    def __unicode__(self):
        return '%s : %s : %s' % (self.newsletter.__unicode__(),
                                 self.contact.__unicode__(),
                                 self.get_status_display())

    class Meta:
        ordering = ('-creation_date',)
        verbose_name = _('contact mailing status')
        verbose_name_plural = _('contact mailing statuses')
        unique_together = ('contact', 'newsletter')

class LinkContactMailingStatus(DateModel):
    contact_mailing_status = models.ForeignKey(ContactMailingStatus)
    link = models.ForeignKey(
        Link,
        related_name = 'link_status',
    )
    click = models.IntegerField(default=0)

    class Meta:
        verbose_name = _('link contact mailing status')
        verbose_name_plural = _('links contact mailing status')

    #class Meta:
    #ordering = ('ordering',)


class WorkGroup(models.Model):
    """Work Group for privatization of the ressources"""
    name = models.CharField(_('name'), max_length=255)
    group = models.ForeignKey(Group, verbose_name=_('permissions group'))

    contacts = models.ManyToManyField(Contact, verbose_name=_('contacts'),
                                      blank=True, null=True)
    mailinglists = models.ManyToManyField(MailingList, verbose_name=_('mailing lists'),
                                          blank=True, null=True)
    newsletters = models.ManyToManyField(Newsletter, verbose_name=_('newsletters'),
                                         blank=True, null=True)

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('workgroup')
        verbose_name_plural = _('workgroups')

class Settings(JSettings):
    pass
